#include "bocone.h"
#include "globprep.h"
#include "globalt.h"
#include "intools.h"
#include "mathem.h"
#include <string.h>

bocone::bocone()
{
  lcid = 0;
  nbc  = 0;
  bct  = NULL;
  pv   = NULL;
  tpv  = NULL;
  trc  = NULL;
  trr  = NULL;
  ccid = NULL;
}

bocone::~bocone()
{
  delete tpv;
}



/**
  Function reads data for boundary condition on elements from the opened text 
  file given by parameter in.
  Parameters :
  @param in  - pointer to opened text file from which data will be read.
  @param nlc - number of load cases.
  Return :
  @retval  0 - on succes
  @retval  1 - error reading boundary condition type
  @retval  2 - unknown type of problem is required
  @retval  3 - unknown type of boundary condition
  @retval  4 - error reading prescribed value for Dirichlet condition
  @retval  5 - error reading flux value for Neumann condition
  @retval  6 - error reading climate condition index
  @retval  7 - error reading transmition
  @retval  8 - error reading special transmition
  @retval  9 - error reading load case id
  @retval 10 - error reading surface id
*/
long bocone::read(FILE *in, long nlc)
{
  long i,ret = 0;
  
  if (getlong(in, lcid))  // reading load case id
  {
    fprintf(stderr, "\n\n Cannot read load case id in function bocone::read\n");
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 9;
  }
  if ((lcid > nlc) || (nlc < 1))
  {
    fprintf(stderr, "\n\n Wrong load case id, it is %ld and it should be in <1,%ld>\n", lcid, nlc);
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 9;
  }
  if (getlong(in, nbc))  // reading surface id
  {
    fprintf(stderr, "\n\n Cannot read number of boundary conditions in function bocone::read\n");
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 10;
  }
  if ((nbc < 1) || (nbc > 6))
  {
    fprintf(stderr, "\n\n Wrong surface id, it is %ld and it should be in <1,6>\n", nbc);
    fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
    return 9;
  }
  bct = new long[nbc];
  pv = new double[nbc];
  ccid = new long[nbc];
  trc = new double[nbc];
  trr = new double[nbc];
  tpv = new gfunct *[nbc];
  memset(bct, 0, nbc*sizeof(*bct));
  memset(pv, 0, nbc*sizeof(*pv));
  memset(ccid, 0, nbc*sizeof(*ccid));
  memset(trc, 0, nbc*sizeof(*trc));
  memset(trr, 0, nbc*sizeof(*trr));
  memset(tpv, 0, nbc*sizeof(*tpv));
  for (i=0; i < nbc; i++)
  {
    if (getlong(in, bct[i])) // reading type of boundary condition
    {
      fprintf(stderr, "\n\n Cannot read type of boundary condition in function bocone::read\n");
      fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
      return 1;
    }
  }
  
  for (i = 0; i < nbc; i++)
  {
    if (bct[i])
    {
      switch (Tp->tprob)
      {
        case stationary_problem :
          ret = read_statpr(in, i);
          break;
        case nonstationary_problem :
        case nonlinear_nonstationary_problem :
          ret = read_nonstatpr(in, i);
          break;
        default :
          ret = 2;
      }
    }
  }
  fflush(stderr);
  switch(ret)
  {  
    case 0:
      break;
    case 2:
      fprintf(stderr, "\n\n Unknown type of problem required in function bocone::read\n");
      fprintf(stderr, "in file %s, line %d\n", __FILE__, __LINE__);
      break;
    case 3:
      fprintf(stderr, "\n\n Unknown type of boundary condition is required\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 4:
      fprintf(stderr, "\n\n Error reading prescribed value\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 5:
      fprintf(stderr, "\n\n Error reading flux\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 6:
      fprintf(stderr, "\n\n Error reading climate condition index\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 7:
      fprintf(stderr, "\n\n Error reading transmition\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    case 8:
      fprintf(stderr, "\n\n Error reading special transmition\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
    default :
      fprintf(stderr, "\n\n Error reading boundary condion on element\n");
      fprintf(stderr, " in function bocone::read, (file %s, line %d)", __FILE__, __LINE__);
      break;
  }
  return ret;
}



/**
  Function reads boundary conditions on elements for stationary problem from the text 
  file given by the parameter in.
  Parameters :
  @param in - pointer to opened text file from which data will be read.
  @param i  - boundary id.
  Return :
  @retval 0 - on succes
  @retval 3 - unknown type of boundary condition
  @retval 4 - error reading prescribed value for Dirichlet condition
  @retval 5 - error reading flux value for Neumann condition
  @retval 6 - error reading climate condition index
  @retval 7 - error reading transmition
  @retval 8 - error reading special transmition
*/
long bocone::read_statpr(FILE *in, long id)
{
  switch(bct[id])
  {
    case 0:  // no condition
      break;
    case 1: // Dirichlet condition
      if (getdouble(in, pv[id]))   return(4);  // reading prescribed values
      break;
    case 2: // Neumann condition
      if (getdouble(in, pv[id]))   return(5);  // reading flux
      break;
    case 4: // climate condition
      if (getlong(in, ccid[id]))   return(6);  // reading condition id
      break;
    case 30: // transmition
      if (getdouble(in, pv[id]))   return(7);   // reading transmition
      if (getdouble(in, trc[id]))  return(7);   // reading transmition coefficient
      break;
    case 90: // transmition special
      if (getdouble(in, pv[id]))   return(8);   // reading transmition
      if (getdouble(in, trc[id]))  return(8);   // reading transmition coefficient
      if (getdouble(in, trr[id]))  return(8);   // reading another transmition coefficient
      break;
    default :
      return (3);
  }
  return 0;
}



/**
  Function reads boundary conditions on elements for nonstationary problem from the text 
  file given by the parameter in.
  Parameters :
  @param in - pointer to opened text file from which data will be read.
  @param i  - boundary id.
  Return :
  @retval 0 - on succes
  @retval 3 - unknown type of boundary condition
  @retval 4 - error reading prescribed value for Dirichlet condition
  @retval 5 - error reading flux value for Neumann condition
  @retval 6 - error reading climate condition index
  @retval 7 - error reading transmition
  @retval 8 - error reading special transmition
*/
long bocone::read_nonstatpr(FILE *in, long id)
{
  tpv[id] = new gfunct;
  switch(bct[id])
  {
    case 0:  // no condition
      break;
    case 1: // Dirichlet condition
      if (tpv[id]->read_prop(in))   return(4);  // reading prescribed values
      break;
    case 2: // Neumann condition
      if (tpv[id]->read_prop(in))   return(5);  // reading flux
      break;
    case 4: // climate condition
      if (getlong(in, ccid[id]))   return(6);  // reading condition id
      break;
    case 30: // transmition
      if (tpv[id]->read_prop(in))  return(7);   // reading transmition
      if (getdouble(in, trc[id]))  return(7);   // reading transmition coefficient
      break;
    case 90: // transmition special
      if (tpv[id]->read_prop(in))  return(8);   // reading transmition
      if (getdouble(in, trc[id]))  return(8);   // reading transmition coefficient
      if (getdouble(in, trr[id]))  return(8);   // reading another transmition coefficient
      break;
    default :
      return (3);
  }
  return 0;
}
