#include <string.h>
#include "intools.h"
#include "iotools.h"
#include "siftop.h"
#include "import_T3d.h"
#include "aliast.h"
#include "globalt.h"

#include "input.h"
#include "bocon.h"
#include "inicon.h"
#include "edgeload.h"
#include "sourceload.h"
#include "bocone.h"
#include "surfboc.h"
#include "dbcrs.h"
#include "dbmat.h"
#include "aggregator.h"


siftop    *Top;  ///< Sifel topology
long Npropn; ///< number of entries with nodal property
long Nprope; ///< number of entries with element property
// Folowing arrays have size equal to number of lines with property description
//  of nodes(elements).
// This numbers are specified in the input file on the first line before section
//  described nodals or elements property
bocon      **Boc;   ///< array of pointers to the nodal boundary conditions structures
long       **Boccn; /**< array with bocon squence number (0 : if Boc[i] == NULL)
                         (positive number : if Boc[i] != NULL)*/
long        *Nnp;   ///< property numbers for nodes
long        *Nep;   ///< property numbers for elements
elemtypet   *Tel;   ///< array of element types
long        *Sel;   ///< array of shape element number
long        *Nmel;  ///< number of materials on the element
mattypet   **Mel;   ///< array of material types
long       **Melid; ///< array of material indeces
crsectypet  *Ncs;   ///< array of cross-section types in the nodes
long        *Ncsi;  ///< array of cross-section indeces in the nodes
crsectypet  *Ecs;   ///< array of cross-section types in the elements
long        *Ecsi;  ///< array of cross-section indeces in the elements
dbmat       *Dbmat; ///< database of materials and their parameters
dbcrs       *Dbcrs; ///< database of cross-sections and their parameters
elemtypet   *Tet;   ///< array of element type for each element
long        *Nmu;   ///< array with indicators of used material types from material database
long        *Ncu;   ///< array with indicators of used cross-section types from cross-section database
long         Nlc;   ///< number of loading cases
long         Ndofn; ///< number of degree of freedom of given problem
long         Nspv;  ///< number of prescribed values
long         Csrin; ///< number of renumbered index in function wr_nodes
long       **Nccn;  ///< array of pointers to common code numbers in the nodes
inicon     **Inic;  ///< array of pointers to the nodal initial conditions structures
edgeload   **Edgl;  ///< array of pointers to the edge load structures
sourceload **Sl;    ///< array of pointers to the edge load structures
bocone     **Boce;  ///< boundary conditions on elements
surfboc    **Surfbc; ///< array of pointers to boundary conditions on surfaces
gfunct      *Tf;    ///< array of time functions used in time dependent mechanical problems for each load case

/**
 Function   reads problem data from file in a writes it to file out in SIFEL format.

 @param in   - pointer to opened input file with problem description
 @param outf - pointer to string with name of output file

 @retval 0 - on succes
 @retval 1 - unable to open temporary file
 @retval 2 - if fails reading number of loading cases
 @retval 3 - if fails reading topology
 @retval 4 - if fails reading materials
 @retval 5 - if fails reading cross-sections
 @retval 6 - if fails reading property file
 @retval 7 - if fails writing output file
*/
long input(XFILE *in, descrip &d)
{
  long npropn, nprope;
  long err, i;
  XFILE *itop;

  // reading number of loading cases
  xfscanf(in, "%ld", &Nlc);
  //  allocation of array of time functions
  Tf = new gfunct [Nlc];
  for (i=0;i<Nlc;i++)
    Tf[i].read (in);

  Ndofn = Tp->ntm;
  
  if (Tp->ssle->prec.pt==boss){
    //  iterative method is preconditioned by the BOSS algorithm
    if (Tp->ssle->prec.agg->impl==2){
      //  the BOSS method is based on the METIS decomposition
      d.paral=2;
    }
  }
  
  if (Tp->ssle->tlinsol==sfeti){
    //  the problem is solved by the FETI method on a single processor
    d.paral=2;
  }

  // reading of topology
  itop = xfopen(d.topf, "rt");
  if (itop == NULL)
    return(3);
  itop->warning = 1;
  itop->kwdmode = ignore;
  //in->kwdmode = sequent_mode;
  itop->ignorecase = 1;

  // reading topology
  err = input_siftop(itop, d);
  if (err)
    return(3);

  // reading material database
  err = input_materials(d.matf);
  if (err)
    return(4);

  // reading cross-section elements
  err = input_crs(d.crf);
  if (err)
    return(5);

  // reading nodal and element properties
  err = input_prop(in, npropn, nprope);
  switch (err)
  {
    case 1 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading property type of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 2 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading property ndof of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 3 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " property ndof of node hasn't been entered before proprety, which it needs.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 4 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading boundary condition of nodes fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 5 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading nodal load fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 6 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading cross-section of node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 7 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading transformation vectors of local system in the node fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 8 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading dynamic nodal load fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 9 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading initial nodal conditions fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 10 :
      fprintf(stderr, "\n\nError reading nodal properties\n");
      fprintf(stderr, " reading common nodal code numbers fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 21 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading property type of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 22 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading element type property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 23 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading material property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 24 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading cross-section property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 25 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 26 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading dynamic load property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    case 27 :
      fprintf(stderr, "\n\nError reading element properties\n");
      fprintf(stderr, " reading local coordinates system property of element fails.\n");
      fprintf(stderr, " Program terminated.\n");
      return(6);
    default :
      break;
  }
  Npropn = npropn;
  Nprope = nprope;
  return(0);
}



/**
 Function reads topology file(T3D format) with name stored in in_t3d

 @param in - pointer to opened XFILE
 @param d - structure with input data format description

  Function returns:
  @retval 0 : on success
  @retval 4 : in case of unknown mesh format

  In case of reading alternative file format :
  @retval 1 : on error in reading of node
  @retval 2 : on error in reading of element
  @retval 3 : on error in reading of global node numbers
*/
long input_siftop(XFILE *in, descrip &d)
{
  Top = new siftop;
  long ret; 

  fprintf(stdout, "\n\nReading of mesh topology . . .");
  switch (d.meshfmt)
  {
    case t3d:
      Top->import_t3d(in, d.paral);
      break;
    case sifel:
      ret = Top->read(in, d.paral, d.redgn);
      return(ret);
    default:
      print_err("unknown mesh format is required", __FILE__, __LINE__, __func__);
      return(4);
  }
  return(0);

}



/**
 Function reads material database from file with name stored in the in

 @param in - string with database file name

 @retval 0 - on succes
 @retval 1 - if fails opening file
 @retval 2 - if fails reading number of material types
 @retval 3 - if fails reading material parameters
*/
long input_materials(char *in)
{
  FILE *f = NULL;
  f = fopen(in, "rt");
  if (f == NULL)
  {
    fprintf(stderr, "\nError - unable open material file :\n %s\n", in);
    return(1);
  }
  Dbmat = new dbmat;  // allocating new dbmat class
  if (getlong(f, Dbmat->numt)) // reading number of material types
  {
    fprintf(stderr, "\nError reading number of material types\n");
    fclose(f);
    return(2);
  }
  Dbmat->mat = new matr[Dbmat->numt]; // allocating memory for each material type
  Nmu = new long[Dbmat->numt]; // allocating array with number of used materials
  memset(Nmu, 0, sizeof(*Nmu)*Dbmat->numt);
  for (long i = 0; i < Dbmat->numt; i++)
  {
    // reading each index of material types
    if (Dbmat->mat[i].read(f))
    {
      fclose(f);
      return(3);
    }
  }
  return(0);
}



/**
 Function reads cross-section database from file with name stored in the in

 @param in - string with database file name

 @retval 0 - on succes
 @retval 1 - if fails opening file
 @retval 2 - if fails reading number of cross-section types
 @retval 3 - if fails reading cross-section parameters
*/
long input_crs(char *in)
{
  FILE *f = NULL;
  f = fopen(in, "rt");
  if (f == NULL)
  {
    fprintf(stderr, "\nError - unable open cross-section file :\n %s\n", in);
    return(1);
  }
  Dbcrs = new dbcrs; // allocating new dbcrs class
  if (getlong(f, Dbcrs->numt))  // reading number of cross-section types
  {
    fprintf(stderr, "\nError reading number of cross-section types\n");
    fclose(f);
    return(2);
  }
  Dbcrs->crs = new csec[Dbcrs->numt]; // allocating memory for each cross-section types
  Ncu = new long[Dbcrs->numt];        // allocating array with number used cross-sections
  memset(Ncu, 0, sizeof(*Ncu)*Dbcrs->numt);
  for (long i = 0; i < Dbcrs->numt; i++)
  {
    // reading cross-section properties for given cross-section type
    if (Dbcrs->crs[i].read(f))
    {
      fclose(f);
      return(3);
    }
  }
  return(0);
}



/**
 Function reads property data of nodes and elements fro file in.

 @param in  - poinetr to opened input file with property description
 @param npropn - number of property entries in section of nodal properties
 @param nprope - number of property entries in section of element properties
 @param numl - number of processed lines

 @retval 0  : if succes
 @retval 1  : if fails reading property type of node
 @retval 2  : if fails reading property ndof of node
 @retval 3  : if ndof property of node hasn't been entered before node property,
              which needs ndof.
 @retval 4  : if fails reading boundary condition of nodes
 @retval 5  : if fails reading nodal load
 @retval 6  : if fails reading cross-section of node
 @retval 7  : if fails reading transformation vectors of local system in the node
 @retval 8  : if fails reading dynamic nodal load
 @retval 9  : if fails reading initial condition
 @retval 10 : if fails reading common code numbers in the nodes
 @retval 21 : if fails reading property type of element
 @retval 22 : if fails reading element type property of element
 @retval 23 : if fails reading material property of element
 @retval 24 : if fails reading cross-section property of element
 @retval 25 : if fails reading load property of element
 @retval 26 : if fails reading dynamic load property of element
 @retval 27 : if fails reading local coordinate system property of element
 @retval 31 : if fails reading number nodal property entries
 @retval 32 : if fails reading number element property entries
*/
long input_prop(XFILE *xin, long &npropn, long &nprope)
{
  long nprop, i, j;
  long ret, ccn = 1, ndir, dir;
  proptypet tp;
  inicon  *tic;
  bocon   *tbc;
  long numl;
  FILE *in = xin->file;

  if (getgkwid(xin, 0, numl)) // searchs begin
  {
    xin->line += numl;  
    return(31);
  }
  xin->line += numl;;
  nprop = getnprop(xin, numl);
  if (nprop < 0) // reading number of lines with nodal propreties
    return(31);
  npropn = nprop;
  Nnp = new long[nprop];   // Array with property numbers
  for (i = 0; i < nprop; Nnp[i] = -1, i++);
  Boc = new bocon*[nprop];              // Array with bounadry condition
  memset(Boc, 0, sizeof(*Boc)*nprop);   //   setting each bc pointer to NULL
  Ncs = new crsectypet[nprop];           // Array of cross-section types in the nodes
  memset(Ncs, 0, sizeof(*Ncs)*nprop);   //   setting each cs type to zero
  Ncsi = new long[nprop];               // Array of cross-section indeces
  memset(Ncsi, 0, sizeof(*Ncsi)*nprop); //   setting each cs to zero
  Nccn = new long*[nprop];
  memset(Nccn, 0, sizeof(*Nccn)*nprop);
  Inic = new inicon*[nprop];
  memset(Inic, 0, sizeof(*Inic)*nprop);
  Sl = new sourceload*[nprop];        // Array with source loads structure
  memset(Sl, 0, sizeof(*Sl)*nprop);

  fprintf(stdout, "\nReading nodal properties (number of entries : %ld) . . .", nprop);
  // reading node properties
  for (i = 0; i < nprop; i++)
  {
    // reading property number and type of nodal property (tp)
    if (getlong(in, Nnp[i]))
      return(1);
    tp = proptypet(getpkw(in)+1);
    switch(tp)
    {
      case boundarycondt :  // reading boundary condition and prescribed displacements
      {
        fprintf(stdout, "\n reading boundary condition");
        tbc = new bocon; // allocating temporary boundary condition class
        if ((ret = tbc->read(in, Ndofn))) // reading boundary condition
          return(4);
        Boc[i] = tbc;  // storing pointer to the nodal bc
        break;
      }
      case crosssect :  // reading cross-section in the node
      {
        fprintf(stdout, "\n reading cross-section");
        if (getint(in, (int &)Ncs[i]) || getlong(in, Ncsi[i]))
        // reading cross-section type and index number
          return(6);
        if ((Ncs[i] == 0) || (Ncsi[i] == 0))
          return(6);
        break;
      }
      case initcondt :  // reading initial condition
      {
        fprintf(stdout, "\n reading initial condition");
        tic = new inicon; // allocating temporary initial condition class
        if ((ret = tic->read_prop(in, Ndofn))) // reading initial condition
          return(9);
        Inic[i] = tic;  // storing pointer to the nodal ic
        break;
      }
      case comcodnumt :  // reading common code numbers
      {
        fprintf(stdout, "\n reading common code numbers");
        if (Nccn[i] == NULL)
        {
          Nccn[i] = new long[Ndofn];
          memset(Nccn[i], 0, sizeof(*Nccn[i])*Ndofn);
          if (getlong(in, ndir))
            return(10);
          for (j = 0; j < ndir; j++)
          {
            if (getlong(in, dir))
              return(10);
            if ((dir < 1) || (dir > Ndofn))
              return(10);
            ccn++;
            Nccn[i][j] = ccn;
          }
        }
        else
          return (10);
        break;
      }
      case loadsource :
      {
        fprintf(stdout, "\n reading nodal sources");
        Sl[i] = new sourceload;
        if(Sl[i]->read(in, Nlc))
          return (11);
        break;
      }
      default :
        return(1);
    }
  }
  xin->line += numl;  // actualization of the line number with number of lines with property entries
  getgkwid(xin, 1, numl); // searchs end
  xin->line += numl;


  getgkwid(xin,0,numl); // searchs begin
  xin->line += numl;
  nprop = getnprop(xin, numl);
  if (nprop < 0) // reading number of lines with element propreties
    return(32);
  nprope = nprop;
  Nep = new long[nprop]; // Array with element property numbers
  for (i = 0; i < nprop; Nep[i] = -1, i++); // setting each property number to -1
  Tel = new elemtypet[nprop];  // Array with element types
  memset(Tel, 0, sizeof(*Tel)*nprop);  // setting each type to zero
  Sel = new long[nprop];  // Array with element shape numbers
  memset(Sel, 0, sizeof(*Sel)*nprop);  // setting each type to zero
  Nmel = new long[nprop];  // Array with number of given material types on the element
  memset(Nmel, 0, sizeof(*Nmel)*nprop);  // setting each number of material to zero
  Mel = new mattypet*[nprop];  // Array with material type numbers
  memset(Mel, 0, sizeof(*Mel)*nprop);  // setting each type number to zero
  Melid = new long*[nprop];  // Array with material type indeces
  memset(Melid, 0, sizeof(*Melid)*nprop);  // setting each material index to zero
  Ecs = new crsectypet[nprop];  // Array with cross-section type numbers
  memset(Ecs, 0, sizeof(*Ecs)*nprop);  // setting each cs type number to zero
  Ecsi = new long[nprop];      // Array with cross-section type indeces
  memset(Ecsi, 0, sizeof(*Ecsi)*nprop);  // setting each cs index to zero
  Edgl = new edgeload*[nprop];  // Array with edge loads
  memset(Edgl, 0, sizeof(*Edgl)*nprop);
  Boce = new bocone*[nprop];  // Array with boundary conditions on elements
  memset(Boce, 0, sizeof(*Boce)*nprop);
  Surfbc = new surfboc*[nprop];  // Array with boundary conditions on surfaces
  memset(Surfbc, 0, sizeof(*Surfbc)*nprop);

  fprintf(stdout, "\nReading element properties (number of entries : %ld) . . .", nprop);
  // reading element properties
  for (i = 0; i < nprop; i++)
  {
    if (getlong(in, Nep[i])) // reading property number and type
      return(21);
    tp = proptypet(getpkw(in)+1);
    switch(tp)
    {
      case eltypet :  // reading element type
      {
        fprintf(stdout, "\n reading element type");
        getlong(in, Sel[i]); // reading element shape number
        getint(in, (int &)Tel[i]); // reading element type number
        if (Tel[i] < 0)
          return(22);
        break;
      }
      case matelt :  // reading element material
      {
        fprintf(stdout, "\n reading material");
        getlong(in, Nmel[i]); // reading number of given material types
        if (Nmel[i] == 0)
          return(23);
        Mel[i]   = new mattypet[Nmel[i]]; // Allocating material type array
        memset(Mel[i], 0, sizeof(*Mel[i])*Nmel[i]); // setting each type to zero
        Melid[i] = new long [Nmel[i]];   // Allocating material type index array
        memset(Melid[i], 0, sizeof(*Melid[i])*Nmel[i]); // setting each index to zero
        for (j = 0; j < Nmel[i]; j++)
        {
          // reading each material type number and index number
          getint(in, (int &)Mel[i][j]);
          getlong(in, Melid[i][j]);
          if ((Mel[i][j] == 0) || (Melid[i][j] == 0))
            return(23);
        }
        break;
      }
      case crosssect :  // reading element cross-section
      {
        fprintf(stdout, "\n reading cross-section");
        // reading cs type number and cs index number
        getint(in, (int &)Ecs[i]);
        getlong(in, Ecsi[i]);
        if ((Ecs[i] == 0) || (Ecsi[i] == 0))
          return(24);
        break;
      }
      case loadedget :
      {
        fprintf(stdout, "\n reading element edge load");
        Edgl[i] = new edgeload;
        if(Edgl[i]->read(in, Nlc))
          return (26);
        break;
      }
      case boundarycondt :
      {
        fprintf(stdout, "\n reading element boundary condition");
        Boce[i] = new bocone;
        if(Boce[i]->read(in, Nlc))
          return (27);
        break;
      }
      case boconsurf :
      {
        fprintf(stdout, "\n reading surface boundary condition");
        Surfbc[i] = new surfboc;
        if(Surfbc[i]->read(in, Nlc))
          return (28);
        break;
      }
      default :
        return(29);
    }
  }
  xin->line += numl;  // actualization of the line number with number of lines with property entries
  getgkwid(xin, 1,numl); // searchs end
  return(0);
}
