/*
    File:             sejtkrmat.cpp
    Author:           Tomas Krejci, 12/09/2008, revised 03/03/2010
    Purpose:          material model for saturated-nonsaturated one-phase flow in deforming medium
    sources:          Lewis and Schrefler's book pp. 86-89, material parameters are set for benchmark on page n. 168
    unknowns:         number of unknowns=1, pw = liqud water pressure
*/ 

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "constrel.h"
#include "sejtkrmat.h"
#include "globalt.h"

sejtkrmat::sejtkrmat()
{
  //
  emod = 1.3e6;//Young's modulus [Pa]
  //
  nu = 0.4;//Poisson's ratio [-]
  //
  alpha = 1.0;//Biot's constant [-] alpha = 1 - kt/ks
  //
  ks = 2.167e6;//bulk modulus of solid phase (grains) [Pa]
  //
  kw = 2.0e9;//bulk modulus of water [Pa]
  //
  //ka = 0.1e6;//bulk modulus of air [Pa]
  //
  phi0 = 0.297;//initial porosity [-]
  //
  kintr = 4.5e-13;//intrinsic permeability [m^2]
  //
  rhos = 2000.0;//solid grain density [kg/m^3]
  //
  rhow = 1000.0;//water density [kg/m^3]
  //
  muw0 = 1.0e-3;//water viscosity [Pa.s]
  //
  //mua = 1.8e-5;//air viscosity [Pa.s]
}

sejtkrmat::~sejtkrmat()
{}


/**
   function reads parameters
   
   @param in - input file

   12/9/2008, TKr
*/
void sejtkrmat::read(XFILE *in)
{
  //xfscanf (in,"%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf", &emod, &nu, &rhos, &alpha, &ks, &phi0, &kw, &rhow, &muw0, &kintr);
}




/**
   function computes volume part of right-hand side matrix
   in the required integration point
   
   @param d - right-hand side %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void sejtkrmat::rhs_volume (matrix &d,long ri,long ci,long ipp)
{
  long m;
  m = d.m;
  
  switch (m){
  case 1:{
    rhs1d1 (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    rhs2d1 (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    rhs3d1 (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    fprintf (stderr,"\n unknown number of components of stress tensor is required");
    fprintf (stderr,"\n in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function creates volume right-hand side matrix of the material for 1D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void sejtkrmat::rhs1d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  f = get_fw1(pw);
  g = Tp->gr1;//vector of gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g;
}

/**
   function creates volume right-hand side matrix of the material for 2D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void sejtkrmat::rhs2d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  g = new double [2];
  f = get_fw1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
}

/**
   function creates volume right-hand side matrix of the material for 3D problems
   @param d - right-hand %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
*/
void sejtkrmat::rhs3d1 (matrix &d,long ri,long ci,long ipp)
{
  double f,pw,pg;
  double *g;
  
  pw = Tm->ip[ipp].av[0];// *scale_pc;//scaling
  
  g = new double [3];
  f = get_fw1(pw);
  
  g[0] = Tp->gr1;//gravity acceleration
  g[1] = Tp->gr2;//gravity acceleration
  g[2] = Tp->gr3;//gravity acceleration
  
  fillm(0.0,d);
  d[0][0] = f*g[0];
  d[1][0] = f*g[1];
  d[2][0] = f*g[2];
}




/**
   function computes degree of saturation(sorption curve)
   @param pw - water pressure

   @retval sw - degree of saturation

   12/9/2008, TKr
*/
double sejtkrmat::get_sw(double pw)
{
  double sw;
  //
  sw = 1.0 - (1.9722e-11)*pow(-pw,2.4279);//p. 167

  return(sw);
}


/**
   function computes partial derivative of degree of saturation with respect to pw, specific wter content
   @param pw - water pressure

   @retval dsw_dpw - partial derivative of degree of saturation with respect to pw

   12/9/2008, TKr
*/
double sejtkrmat::get_dsw_dpw(double pw)
{
  double dsw_dpw;
  //

  dsw_dpw = 2.4279*(1.9722e-11)*pow(-pw,(2.4279-1.0));//p. 167

  return(dsw_dpw);
}


/**
   function computes water relative permeability
   @param pw - water pressure
   
   @retval krw - water relative permeability

   12/9/2008, TKr
*/
double sejtkrmat::get_krw(double pw)
{
  double krw,s;
  
  s = get_sw(pw);
  krw = 1.0+2207*pow((1.0-s),1.0121); //p. 167

  return(krw);
}


/**
   function returns porosity

   @retval phi - porosity

   12/9/2008, TKr
*/
double sejtkrmat::get_phi()
{
  double phi;

  phi = phi0;

  return(phi);
}


/**
   function computes intrinsic permeability

   @retval kintr - intrinsic permeability

   12/9/2008, TKr
*/
double sejtkrmat::get_kintr()
{
  return(kintr);
}


/**
   function returns volume density of soil skeleton

   @retval rhos - volume density of concrete skeleton

   12/9/2008, TKr
*/
double sejtkrmat::get_rhos()
{
  return(rhos);
}


/**
   function returns Biot's constant

   @retval alpha - Biot's constant

   12/9/2008, TKr
*/
double sejtkrmat::get_alpha()
{
  return(alpha);
}


/**
   function returns emod Young's modulus

   @retval emod = Young's modulus

   12/9/2008, TKr
*/
double sejtkrmat::get_emod()
{
  return(emod);
}

/**
   function returns nu = Poisson's ratio

   @retval nu - Poisson's ratio

   12/9/2008, TKr
*/
double sejtkrmat::get_nu()
{
  return(nu);
}


/**
   function returns ks bulk modulus of solid phase (grains) [Pa]

   @retval ks - bulk modulus of solid phase (grains) [Pa]

   12/9/2008, TKr
*/
double sejtkrmat::get_ks()
{
  return(ks);
}


/**
   function returns kw bulk modulus of water [Pa]

   @retval kw - bulk modulus of water [Pa]

   12/9/2008, TKr
*/
double sejtkrmat::get_kw()
{
  return(kw);
}


/**
   function returns muw water viscosity [Pa.s]

   @retval muw - water viscosity [Pa.s]

   12/9/2008, TKr
*/
double sejtkrmat::get_muw()
{
  return(muw0);
}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kuw - conductivity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_kuw(double pw)
{
  double s,kuw;
  
  s = get_sw(pw);

  kuw = -alpha*s;//p. 88

  return(kuw);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kwu - conductivity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_kwu(double pw)
{
  double kwu;

  kwu = 0.0;

  return(kwu);

}


/**
   function creates conductivity coefficient of the general material
   @param pw - water pressure

   @retval kww - conductivity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_kww(double pw)
{
  double krw,kww;
  
  krw = get_krw(pw);

  kww = krw*kintr/muw0;//p. 89

  //printf("\n pw = %e, kww = %e",pw,kww);

  return(kww);
}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capuw - capacity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_capuw(double pw)
{
  double capuw;
  
  capuw = 0.0;

  return(capuw);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capwu - capacity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_capwu(double pw)
{
  double s,capwu;
  
  s = get_sw(pw);

  capwu = alpha*s;//p. 88

  return(capwu);

}


/**
   function creates capacity coefficient of the general material
   @param pw - water pressure

   @retval capww - capacity coefficient

   12/9/2008, TKr
*/
double sejtkrmat::get_capww(double pw)
{
  double sw,dsw_dpw,n,capww;  
  
  sw = get_sw(pw);
  dsw_dpw = get_dsw_dpw(pw);
  n = phi0;
  
  //capww = (alpha-n)/ks*sw*(sw + dsw_dpw/n*pw)+n*sw/kw + dsw_dpw;//p. 89 this equation is wrong
  capww = (alpha-n)/ks*sw*(sw + dsw_dpw*pw)+n*sw/kw + n*dsw_dpw;//this equation is correct (different derivation)
  //capww = (alpha-n)/ks+n*sw/kw;//fully saturation state only for debug
  
  //printf("\n pw = %e, capww = %e",pw,capww);

  return(capww);

}


/**
   function returns coefficient for righ-hand side of the general material
   @param pw - water pressure

   @retval fw - first part for right-hand side for continutiy equation

   12/9/2008, TKr
*/
double sejtkrmat::get_fw1(double pw)
{
  double krw,fw1;
  
  krw = get_krw(pw);

  fw1 = krw*kintr/muw0*rhow;//p. 89

  //printf("\n pw = %e, fw1 = %e",pw,fw1);

  return(fw1);

}



/**
   function returns coefficient for righ-hand side of the general material 
   @param pw - water pressure
   @param pg - air pressure

   @retval fu1 - first part for right-hand side for balance equation

   12/9/2008, TKr
*/
double sejtkrmat::get_fu1(double pw)
{
  double sw,n,fu1;  
  
  sw = get_sw(pw);
  n = phi0;
  
  fu1 = rhos*(n-1) + sw*n*rhow;//p. 88

  return(fu1);

}

/**
   function computes all other variables at nodes
   @param compother - number of other components
   @param pw - water capillary pressure on actual node
   @param pg - gas(air) pressure on actual node
   @param ipp - first integration point on element

   @retval other - other variable

   03/03/2011, TKr
*/

double sejtkrmat::get_othervalue(long compother,double pw, long ipp)
{
  double other;
  state_eq tt;

  switch (compother){
  case 0:{//capillary pressure
    other = -pw;
      break;
  }
  case 1:{//saturation
    other = get_sw(pw);
    break;
  }
  case 2:{//liquid water pressure
    other = pw;
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
  return (other);

}



/**
     function prints names of all other variables at nodes
     @param out - output rhle
     @param compother - number of other components

     03/03/2011, TKr
*/
void sejtkrmat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//capillary pressure
    fprintf (out,"Capillary pressure (Pa)");
    break;
  }
  case 1:{//saturation
    fprintf (out,"Degree of saturation ()       ");
    break;
  }
  case 2:{//liquid water pressure
    fprintf (out,"Pore water pressure (Pa)      ");
    break;
  }
  default:{
    fprintf (stderr,"\n\n unknown type of component is required in function (%s, line %d).\n",__FILE__,__LINE__);
  }
  }
}



/**
   function checks if computed unknowns are physically reasonable
   @param nv  - vector of unknowns
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void sejtkrmat::values_correction (vector &nv, long ipp)
{
  //  pore water pressure control
  water_pressure_check(nv[0],ipp);
}


/**
   function checks if water pressure is non-positive
   @param pw - pore water pressure
   @param ipp - number of integration point


   28/02/2011, TKr
*/
void sejtkrmat::water_pressure_check(double &pw,long ipp)
{
  if (pw >= 0.0) 
    pw = -1.0;
  
  //storing into gauss point
  Tm->ip[ipp].av[0] = pw;
}
