/*
  File:             homogmat.cpp
  Author:           Jaroslav Kruis, 23.9.2010
  Purpose:          Calculates properties of homogenized material
*/ 

#include "homogmat.h"
#include "globalt.h"

homogmat::homogmat (void)
{
  allocm (4,4,dd);
  allocm (2,2,cc);
}
homogmat::~homogmat (void)
{
  destrm(dd);
  destrm(cc);
}




/**
   function computes conductivity %matrix of the material
   in the required integration point
   
   @param d - conductivity %matrix of material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   JK, 23.9.2010
*/
void homogmat::matcond (matrix &d,long ri,long ci,long ipp)
{
  long n;
  n = d.n;

  switch (n){
  case 1:{
    matcond1d (d,ri,ci,ipp);//1D
    break;
  }
  case 2:{
    matcond2d (d,ri,ci,ipp);//2D
    break;
  }
  case 3:{
    matcond3d (d,ri,ci,ipp);//3D
    break;
  }
  default:{
    print_err("unknown number of components of conductivity tensor is required",__FILE__,__LINE__,__func__);
  }
  }
}


/**
   function creates conductivity %matrix of the material for 1D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   JK, 23.9.2010
*/
void homogmat::matcond1d (matrix &d,long ri,long ci,long ipp)
{
  d[0][0] = dd[ri][ci];
}

/**
   function creates conductivity %matrix of the material for 2D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   JK, 23.9.2010
*/
void homogmat::matcond2d (matrix &d,long ri,long ci,long ipp)
{
  d[0][0] = dd[2*ri+0][2*ci+0];   d[0][1] = dd[2*ri+0][2*ci+1];
  d[1][0] = dd[2*ri+1][2*ci+0];   d[1][1] = dd[2*ri+1][2*ci+1];
}

/**
   function creates conductivity %matrix of the material for 3D problems

   @param d   - conductivity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   JK, 23.9.2010
*/
void homogmat::matcond3d (matrix &d,long ri,long ci,long ipp)
{
  d[0][0] = dd[3*ri+0][3*ci+0];   d[0][1] = dd[3*ri+0][3*ci+1];   d[0][2] = dd[3*ri+0][3*ci+2];
  d[1][0] = dd[3*ri+1][3*ci+0];   d[1][1] = dd[3*ri+1][3*ci+1];   d[1][2] = dd[3*ri+1][3*ci+2];
  d[2][0] = dd[3*ri+2][3*ci+0];   d[2][1] = dd[3*ri+2][3*ci+1];   d[2][2] = dd[3*ri+2][3*ci+2];
}


/**
   function creates capacity %matrix of the material

   @param c   - capacity %matrix of the material
   @param ri  - row index
   @param ci  - column index   
   @param ipp - number of integration point
   
   JK, 23.9.2010
*/
void homogmat::matcap (double &c,long ri,long ci,long ipp)
{
  c = cc[ri][ci];
}

/**
   function assembles the conductivity %matrix after homogenization
   
   @param d - array of %matrix entries
   
   JK, 23.9.2010
*/
void homogmat::assemble_matrices (double *d)
{
  dd[0][0]=d[0];   dd[0][1]=d[1];   dd[0][2]=d[2];   dd[0][3]=d[3];
  dd[1][0]=d[4];   dd[1][1]=d[5];   dd[1][2]=d[6];   dd[1][3]=d[7];
  dd[2][0]=d[8];   dd[2][1]=d[9];   dd[2][2]=d[10];  dd[2][3]=d[11];
  dd[3][0]=d[12];  dd[3][1]=d[13];  dd[3][2]=d[14];  dd[3][3]=d[15];
  
  cc[0][0]=d[16];    cc[0][1]=d[17];
  cc[1][0]=d[18];    cc[1][1]=d[19];
  
  /*
  fprintf (stdout,"\n\n homogmat.cpp\n");
  fprintf (stdout,"\n %le %le %le %le",d[0],d[1],d[2],d[3]);
  fprintf (stdout,"\n %le %le %le %le",d[4],d[5],d[6],d[7]);
  fprintf (stdout,"\n %le %le %le %le",d[8],d[9],d[10],d[11]);
  fprintf (stdout,"\n %le %le %le %le",d[12],d[13],d[14],d[15]);

  fprintf (stdout,"\n %le %le %le %le",d[16],d[17],d[18],d[19]);
*/
}


/**
   function computes new transmission coefficient (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param trc     - prescribed transmission coefficient on the boundary
   @param ri      - row index
   @param ci      - column index
   @param nn      - number of node
   @param bc      - type of boundary condition
   @param ipp     - number of first integration point on element

   TKr, 26/09/2011
*/
double homogmat::transmission_transcoeff(double trc,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double new_trc,w,t;
  new_trc = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {w = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {w = 0.0;}
  if (k<0)   {w = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {t = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {t = 0.0;}
  if (k<0)   {t = Tb->lc[0].pv[0-k-1].getval();}
  
  if((ri == 0) && (ci == 0))
    new_trc = get_transmission_transcoeff_ww(w,t,bc,ipp);
  if((ri == 0) && (ci == 1))
    new_trc = 0.0;
  
  if((ri == 1) && (ci == 0))
    new_trc = 0.0;
  if((ri == 1) && (ci == 1))
    new_trc = get_transmission_transcoeff_tt(w,t,bc,ipp);

  new_trc = new_trc*trc;

  return (new_trc);
}

/**
   function computes new nodal value (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element

   TKr, 26/09/2011
*/
double homogmat::transmission_nodval(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double new_nodval,w,t;
  new_nodval = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {w = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {w = 0.0;}
  if (k<0)   {w = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {t = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {t = 0.0;}
  if (k<0)   {t = Tb->lc[0].pv[0-k-1].getval();}
  
  if((ri == 0) && (ci == 0))
    new_nodval = get_transmission_nodval_ww(nodval,w,t,bc,ipp);
  if((ri == 0) && (ci == 1))
    new_nodval = 0.0;
  
  if((ri == 1) && (ci == 0))
    new_nodval = 0.0;
  if((ri == 1) && (ci == 1))
    new_nodval = get_transmission_nodval_tt(nodval,w,t,bc,ipp);

  return (new_nodval);
}


/**
   function computes flux (for transmission_vector) 
   for boundary condition (third kind of boundary condition)

   @param nodval     - prescribed nodal value
   @param trc2       - second prescribed transmission coefficient on the boundary, 
                       if is needed (for example heat radiation coef.)
   @param ri         - row index
   @param ci         - column index
   @param nn         - number of node
   @param bc         - type of boundary condition
   @param ipp        - number of first integration point on element

   TKr, 26/09/2011
*/
double homogmat::transmission_flux(double nodval,double trc2,long ri,long ci,long nn,long bc,long ipp)
{
  long k;
  double flux,w,t;
  flux = 0.0;
  
  k=Gtt->give_dof(nn,0);
  if (k>0)   {w = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {w = 0.0;}
  if (k<0)   {w = Tb->lc[0].pv[0-k-1].getval();}
  k=Gtt->give_dof(nn,1);
  if (k>0)   {t = Lsrst->lhs[k-1]+Lsrst->lhsi[k-1];}
  if (k==0)  {t = 0.0;}
  if (k<0)   {t = Tb->lc[0].pv[0-k-1].getval();}
  
  if((ri == 0) && (ci == 0))
    flux = get_transmission_flux_ww(nodval,w,t,bc,ipp);
  if((ri == 0) && (ci == 1))
    flux = 0.0;
  
  if((ri == 1) && (ci == 0))
    flux = 0.0;
  if((ri == 1) && (ci == 1))
    flux = get_transmission_flux_tt(nodval,w,t,bc,ipp);

  return (flux);
}


/**
   function creates transfer coefficient on the boundary for prescribed condition (water vapour pressure)

   @param w   - water content
   @param t   - temperature
   @param bc  - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_transcoeff_ww(double w,double t,long bc,long ipp)
{
  double trc;
  double pgws;

  switch (bc){//type of prescribed variable
  case 30:{// relative humidity = physically incorrect b.c.
    trc = 1.0;
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  
  return(trc);
}

/**
   function computes the right prescribed value on the boundary for prescribed condition (water vapour pressure)

   @param bv - prescribed value near the boundary
   @param w - actual water conctent on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_nodval_ww(double bv,double w,double t,long bc,long ipp)
{  
  double nodval;
  double pgws,rh;

  switch (bc){//type of prescribed variable
  case 30:{// relative humidity = physically incorrect b.c.
    nodval = bv;
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  return(nodval);
}


/**
   function creates flux on the boundary (convective mass transfer)

   @param bv - prescribed value near the boundary
   @param w - water content
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_flux_ww(double bv,double w,double t,long bc,long ipp)
{
  double flux;
  double pgws,rh;

  switch (bc){//type of prescribed variable
  case 30:{// relative humidity = physically incorrect b.c.
    flux = (bv - t);
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  return(flux);
}


/**
   function creates transfer coefficient on the boundary for prescribed condition (temperature)

   @param w   - water content
   @param t   - temperature
   @param bc  - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_transcoeff_tt(double w,double t,long bc,long ipp)
{
  double trc;

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission 
    trc = 1.0;
    break;
  }
  case 31:{//heat transmission for testing (and for boundary flux)    
    trc = 0.0;
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  
  return(trc);
}

/**
   function computes the right prescribed value on the boundary for prescribed condition (temperature)

   @param bv - prescribed value near the boundary
   @param w - actual water conctent on the boundary
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_nodval_tt(double bv,double w,double t,long bc,long ipp)
{  
  double nodval;

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission
    nodval = bv;
    break;
  }
  case 31:{//heat transmission for testing (and for boundary flux)
    nodval = (bv - t);
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  return(nodval);
}


/**
   function creates heat flux on the boundary

   @param bv - prescribed value near the boundary
   @param w - water content
   @param t - actual temperature on the boundary
   @param bc - type of boundary condition

   TKr, 26/09/2011
*/
double homogmat::get_transmission_flux_tt(double bv,double w,double t,long bc,long ipp)
{
  double flux;

  switch (bc){//type of prescribed variable
  case 30:{//heat transmission
    flux = (bv - t);
    break;
  }
  case 31:{//heat transmission for testing (and for boundary flux)
    flux = (bv - t);
    break;
  }
  default:{
    print_err("\n\n No real boundary condition is prescribed (%s, line %d).\n",__FILE__,__LINE__,__func__);
    exit(0);
  }
  }
  return(flux);
}


/**
   function computes all variables in nodes
   @param compother - number of other components
   @param ipp - first integration point on element
   @param pc - capillary pressure on actual node
   @param pg - gas pressure on actual node
   @param t - temperature on actual node
*/

double homogmat::get_othervalue(long compother,double rh,double t,long ipp)
{
  double other;
  
  switch (compother){
  case 0:{//relative humidity
    other = rh;
    break;
  }
  case 1:{//temperature
    other = t;
    break;
  }
  default:{
    print_err("\n\n This component type is not implemented\n",__FILE__,__LINE__,__func__);
  }
  }
  return (other);
}

/**
   function prints names of all variables in nodes
   @param out - output file
   @param compother - number of other components
   
   TKr, 26/09/2011
*/
void homogmat::print_othervalue_name(FILE *out,long compother)
{
  switch (compother){
  case 0:{//Relative humidity
    fprintf (out,"Relative humidity ()       ");
    break;
  }
  case 1:{//Temperature
    fprintf (out,"Temperature (K)             ");
    break;
  }
  default:{
    print_err("\n\n This component type is not implemented\n",__FILE__,__LINE__,__func__);
  }
  }
}
